@@ -2,19 +2,11 @@ package cron
 
 import (
 	"log/slog"
-	"time"
 )
 
 // Option represents a modification to the default behavior of a Cron.
 type Option func(*Cron)
 
-// WithLocation overrides the timezone of the cron instance.
-func WithLocation(loc *time.Location) Option {
-	return func(c *Cron) {
-		c.location = loc
-	}
-}
-
 // WithSeconds overrides the parser used for interpreting job schedules to
 // include a seconds field as the first one.
 func WithSeconds() Option {
@@ -44,3 +36,21 @@ func WithLogger(logger *slog.Logger) Option {
 		c.logger = logger
 	}
 }
+
+func WithClock(clock Clock) Option {
+	return func(c *Cron) {
+		options := clock.Register(c)
+		for _, option := range options {
+			option(c)
+		}
+		c.clock = clock
+	}
+}
+
+// WithOnCycleCompleted registers a callback that will be executed every time all jobs executions that
+// have been started in the same instant have completed.
+func WithOnCycleCompleted(f func()) Option {
+	return func(c *Cron) {
+		c.onCycleCompleted = append(c.onCycleCompleted, f)
+	}
+}
