@@ -17,9 +17,8 @@
 package gitlab
 
 import (
-	"fmt"
+	"errors"
 	"net/http"
-	"net/url"
 	"time"
 )
 
@@ -141,7 +140,7 @@ type Commit struct {
 	Stats            *CommitStats      `json:"stats"`
 	Status           *BuildStateValue  `json:"status"`
 	LastPipeline     *PipelineInfo     `json:"last_pipeline"`
-	ProjectID        int               `json:"project_id"`
+	ProjectID        int64             `json:"project_id"`
 	Trailers         map[string]string `json:"trailers"`
 	ExtendedTrailers map[string]string `json:"extended_trailers"`
 	WebURL           string            `json:"web_url"`
@@ -151,9 +150,9 @@ type Commit struct {
 //
 // GitLab API docs: https://docs.gitlab.com/api/commits/
 type CommitStats struct {
-	Additions int `json:"additions"`
-	Deletions int `json:"deletions"`
-	Total     int `json:"total"`
+	Additions int64 `json:"additions"`
+	Deletions int64 `json:"deletions"`
+	Total     int64 `json:"total"`
 }
 
 func (c Commit) String() string {
@@ -177,24 +176,12 @@ type ListCommitsOptions struct {
 }
 
 func (s *CommitsService) ListCommits(pid any, opt *ListCommitsOptions, options ...RequestOptionFunc) ([]*Commit, *Response, error) {
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("projects/%s/repository/commits", PathEscape(project))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var c []*Commit
-	resp, err := s.client.Do(req, &c)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return c, resp, nil
+	return do[[]*Commit](s.client,
+		withMethod(http.MethodGet),
+		withPath("projects/%s/repository/commits", ProjectID{pid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // CommitRef represents the reference of branches/tags in a commit.
@@ -216,24 +203,12 @@ type GetCommitRefsOptions struct {
 }
 
 func (s *CommitsService) GetCommitRefs(pid any, sha string, opt *GetCommitRefsOptions, options ...RequestOptionFunc) ([]*CommitRef, *Response, error) {
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("projects/%s/repository/commits/%s/refs", PathEscape(project), url.PathEscape(sha))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var cs []*CommitRef
-	resp, err := s.client.Do(req, &cs)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return cs, resp, nil
+	return do[[]*CommitRef](s.client,
+		withMethod(http.MethodGet),
+		withPath("projects/%s/repository/commits/%s/refs", ProjectID{pid}, sha),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // GetCommitOptions represents the available GetCommit() options.
@@ -245,27 +220,16 @@ type GetCommitOptions struct {
 }
 
 func (s *CommitsService) GetCommit(pid any, sha string, opt *GetCommitOptions, options ...RequestOptionFunc) (*Commit, *Response, error) {
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
 	if sha == "" {
-		return nil, nil, fmt.Errorf("SHA must be a non-empty string")
-	}
-	u := fmt.Sprintf("projects/%s/repository/commits/%s", PathEscape(project), url.PathEscape(sha))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
+		return nil, nil, errors.New("SHA must be a non-empty string")
 	}
 
-	c := new(Commit)
-	resp, err := s.client.Do(req, c)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return c, resp, nil
+	return do[*Commit](s.client,
+		withMethod(http.MethodGet),
+		withPath("projects/%s/repository/commits/%s", ProjectID{pid}, sha),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // CreateCommitOptions represents the available options for a new commit.
@@ -299,24 +263,12 @@ type CommitActionOptions struct {
 }
 
 func (s *CommitsService) CreateCommit(pid any, opt *CreateCommitOptions, options ...RequestOptionFunc) (*Commit, *Response, error) {
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("projects/%s/repository/commits", PathEscape(project))
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	c := new(Commit)
-	resp, err := s.client.Do(req, &c)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return c, resp, nil
+	return do[*Commit](s.client,
+		withMethod(http.MethodPost),
+		withPath("projects/%s/repository/commits", ProjectID{pid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // Diff represents a GitLab diff.
@@ -347,24 +299,12 @@ type GetCommitDiffOptions struct {
 }
 
 func (s *CommitsService) GetCommitDiff(pid any, sha string, opt *GetCommitDiffOptions, options ...RequestOptionFunc) ([]*Diff, *Response, error) {
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("projects/%s/repository/commits/%s/diff", PathEscape(project), url.PathEscape(sha))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var d []*Diff
-	resp, err := s.client.Do(req, &d)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return d, resp, nil
+	return do[[]*Diff](s.client,
+		withMethod(http.MethodGet),
+		withPath("projects/%s/repository/commits/%s/diff", ProjectID{pid}, sha),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // CommitComment represents a GitLab commit comment.
@@ -373,14 +313,14 @@ func (s *CommitsService) GetCommitDiff(pid any, sha string, opt *GetCommitDiffOp
 type CommitComment struct {
 	Note     string `json:"note"`
 	Path     string `json:"path"`
-	Line     int    `json:"line"`
+	Line     int64  `json:"line"`
 	LineType string `json:"line_type"`
 	Author   Author `json:"author"`
 }
 
 // Author represents a GitLab commit author
 type Author struct {
-	ID        int        `json:"id"`
+	ID        int64      `json:"id"`
 	Username  string     `json:"username"`
 	Email     string     `json:"email"`
 	Name      string     `json:"name"`
@@ -397,27 +337,17 @@ func (c CommitComment) String() string {
 //
 // GitLab API docs:
 // https://docs.gitlab.com/api/commits/#get-the-comments-of-a-commit
-type GetCommitCommentsOptions ListOptions
+type GetCommitCommentsOptions struct {
+	ListOptions
+}
 
 func (s *CommitsService) GetCommitComments(pid any, sha string, opt *GetCommitCommentsOptions, options ...RequestOptionFunc) ([]*CommitComment, *Response, error) {
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("projects/%s/repository/commits/%s/comments", PathEscape(project), url.PathEscape(sha))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var c []*CommitComment
-	resp, err := s.client.Do(req, &c)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return c, resp, nil
+	return do[[]*CommitComment](s.client,
+		withMethod(http.MethodGet),
+		withPath("projects/%s/repository/commits/%s/comments", ProjectID{pid}, sha),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // PostCommitCommentOptions represents the available PostCommitComment()
@@ -428,29 +358,17 @@ func (s *CommitsService) GetCommitComments(pid any, sha string, opt *GetCommitCo
 type PostCommitCommentOptions struct {
 	Note     *string `url:"note,omitempty" json:"note,omitempty"`
 	Path     *string `url:"path" json:"path"`
-	Line     *int    `url:"line" json:"line"`
+	Line     *int64  `url:"line" json:"line"`
 	LineType *string `url:"line_type" json:"line_type"`
 }
 
 func (s *CommitsService) PostCommitComment(pid any, sha string, opt *PostCommitCommentOptions, options ...RequestOptionFunc) (*CommitComment, *Response, error) {
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("projects/%s/repository/commits/%s/comments", PathEscape(project), url.PathEscape(sha))
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	c := new(CommitComment)
-	resp, err := s.client.Do(req, c)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return c, resp, nil
+	return do[*CommitComment](s.client,
+		withMethod(http.MethodPost),
+		withPath("projects/%s/repository/commits/%s/comments", ProjectID{pid}, sha),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // GetCommitStatusesOptions represents the available GetCommitStatuses() options.
@@ -461,7 +379,7 @@ type GetCommitStatusesOptions struct {
 	Ref        *string `url:"ref,omitempty" json:"ref,omitempty"`
 	Stage      *string `url:"stage,omitempty" json:"stage,omitempty"`
 	Name       *string `url:"name,omitempty" json:"name,omitempty"`
-	PipelineID *int    `url:"pipeline_id,omitempty" json:"pipeline_id,omitempty"`
+	PipelineID *int64  `url:"pipeline_id,omitempty" json:"pipeline_id,omitempty"`
 	All        *bool   `url:"all,omitempty" json:"all,omitempty"`
 }
 
@@ -469,7 +387,7 @@ type GetCommitStatusesOptions struct {
 //
 // GitLab API docs: https://docs.gitlab.com/api/commits/#commit-status
 type CommitStatus struct {
-	ID           int        `json:"id"`
+	ID           int64      `json:"id"`
 	SHA          string     `json:"sha"`
 	Ref          string     `json:"ref"`
 	Status       string     `json:"status"`
@@ -479,31 +397,19 @@ type CommitStatus struct {
 	Name         string     `json:"name"`
 	AllowFailure bool       `json:"allow_failure"`
 	Coverage     float64    `json:"coverage"`
-	PipelineId   int        `json:"pipeline_id"`
+	PipelineID   int64      `json:"pipeline_id"`
 	Author       Author     `json:"author"`
 	Description  string     `json:"description"`
 	TargetURL    string     `json:"target_url"`
 }
 
 func (s *CommitsService) GetCommitStatuses(pid any, sha string, opt *GetCommitStatusesOptions, options ...RequestOptionFunc) ([]*CommitStatus, *Response, error) {
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("projects/%s/repository/commits/%s/statuses", PathEscape(project), url.PathEscape(sha))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var cs []*CommitStatus
-	resp, err := s.client.Do(req, &cs)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return cs, resp, nil
+	return do[[]*CommitStatus](s.client,
+		withMethod(http.MethodGet),
+		withPath("projects/%s/repository/commits/%s/statuses", ProjectID{pid}, sha),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // SetCommitStatusOptions represents the available SetCommitStatus() options.
@@ -517,49 +423,24 @@ type SetCommitStatusOptions struct {
 	TargetURL   *string         `url:"target_url,omitempty" json:"target_url,omitempty"`
 	Description *string         `url:"description,omitempty" json:"description,omitempty"`
 	Coverage    *float64        `url:"coverage,omitempty" json:"coverage,omitempty"`
-	PipelineID  *int            `url:"pipeline_id,omitempty" json:"pipeline_id,omitempty"`
+	PipelineID  *int64          `url:"pipeline_id,omitempty" json:"pipeline_id,omitempty"`
 }
 
 func (s *CommitsService) SetCommitStatus(pid any, sha string, opt *SetCommitStatusOptions, options ...RequestOptionFunc) (*CommitStatus, *Response, error) {
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("projects/%s/statuses/%s", PathEscape(project), url.PathEscape(sha))
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	cs := new(CommitStatus)
-	resp, err := s.client.Do(req, &cs)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return cs, resp, nil
+	return do[*CommitStatus](s.client,
+		withMethod(http.MethodPost),
+		withPath("projects/%s/statuses/%s", ProjectID{pid}, sha),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *CommitsService) ListMergeRequestsByCommit(pid any, sha string, options ...RequestOptionFunc) ([]*BasicMergeRequest, *Response, error) {
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("projects/%s/repository/commits/%s/merge_requests", PathEscape(project), url.PathEscape(sha))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var mrs []*BasicMergeRequest
-	resp, err := s.client.Do(req, &mrs)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return mrs, resp, nil
+	return do[[]*BasicMergeRequest](s.client,
+		withMethod(http.MethodGet),
+		withPath("projects/%s/repository/commits/%s/merge_requests", ProjectID{pid}, sha),
+		withRequestOpts(options...),
+	)
 }
 
 // CherryPickCommitOptions represents the available CherryPickCommit() options.
@@ -571,53 +452,28 @@ type CherryPickCommitOptions struct {
 	Message *string `url:"message,omitempty" json:"message,omitempty"`
 }
 
-func (s *CommitsService) CherryPickCommit(pid any, sha string, opt *CherryPickCommitOptions, options ...RequestOptionFunc) (*Commit, *Response, error) {
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("projects/%s/repository/commits/%s/cherry_pick", PathEscape(project), url.PathEscape(sha))
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	c := new(Commit)
-	resp, err := s.client.Do(req, &c)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return c, resp, nil
-}
-
 // RevertCommitOptions represents the available RevertCommit() options.
-//
 // GitLab API docs: https://docs.gitlab.com/api/commits/#revert-a-commit
 type RevertCommitOptions struct {
 	Branch *string `url:"branch,omitempty" json:"branch,omitempty"`
 }
 
+func (s *CommitsService) CherryPickCommit(pid any, sha string, opt *CherryPickCommitOptions, options ...RequestOptionFunc) (*Commit, *Response, error) {
+	return do[*Commit](s.client,
+		withMethod(http.MethodPost),
+		withPath("projects/%s/repository/commits/%s/cherry_pick", ProjectID{pid}, sha),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
+}
+
 func (s *CommitsService) RevertCommit(pid any, sha string, opt *RevertCommitOptions, options ...RequestOptionFunc) (*Commit, *Response, error) {
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("projects/%s/repository/commits/%s/revert", PathEscape(project), url.PathEscape(sha))
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	c := new(Commit)
-	resp, err := s.client.Do(req, &c)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return c, resp, nil
+	return do[*Commit](s.client,
+		withMethod(http.MethodPost),
+		withPath("projects/%s/repository/commits/%s/revert", ProjectID{pid}, sha),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // GPGSignature represents a GitLab commit's GPG Signature.
@@ -625,31 +481,18 @@ func (s *CommitsService) RevertCommit(pid any, sha string, opt *RevertCommitOpti
 // GitLab API docs:
 // https://docs.gitlab.com/api/commits/#get-signature-of-a-commit
 type GPGSignature struct {
-	KeyID              int    `json:"gpg_key_id"`
+	KeyID              int64  `json:"gpg_key_id"`
 	KeyPrimaryKeyID    string `json:"gpg_key_primary_keyid"`
 	KeyUserName        string `json:"gpg_key_user_name"`
 	KeyUserEmail       string `json:"gpg_key_user_email"`
 	VerificationStatus string `json:"verification_status"`
-	KeySubkeyID        int    `json:"gpg_key_subkey_id"`
+	KeySubkeyID        int64  `json:"gpg_key_subkey_id"`
 }
 
 func (s *CommitsService) GetGPGSignature(pid any, sha string, options ...RequestOptionFunc) (*GPGSignature, *Response, error) {
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("projects/%s/repository/commits/%s/signature", PathEscape(project), url.PathEscape(sha))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	sig := new(GPGSignature)
-	resp, err := s.client.Do(req, &sig)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return sig, resp, nil
+	return do[*GPGSignature](s.client,
+		withMethod(http.MethodGet),
+		withPath("projects/%s/repository/commits/%s/signature", ProjectID{pid}, sha),
+		withRequestOpts(options...),
+	)
 }
