@@ -54,6 +54,7 @@ type Opts struct {
 }
 
 type client struct {
+	forgeID                      int64
 	url                          string
 	urlAPI                       string
 	clientID                     string
@@ -66,8 +67,9 @@ type client struct {
 
 // New returns a Forge implementation that integrates with Bitbucket DataCenter/Server,
 // the on-premise edition of Bitbucket Cloud, formerly known as Stash.
-func New(opts Opts) (forge.Forge, error) {
+func New(id int64, opts Opts) (forge.Forge, error) {
 	config := &client{
+		forgeID:                      id,
 		url:                          opts.URL,
 		urlAPI:                       fmt.Sprintf("%s/rest", opts.URL),
 		clientID:                     opts.OAuthClientID,
@@ -190,12 +192,12 @@ func (c *client) Repo(ctx context.Context, u *model.User, rID model.ForgeRemoteI
 			opts.Start = resp.NextPageStart
 		}
 		if repo == nil {
-			return nil, fmt.Errorf("unable to find repository with id: %s", rID)
+			return nil, fmt.Errorf("%w: unable to find repository with id: %s", forge_types.ErrRepoNotFound, rID)
 		}
 	} else {
 		repo, _, err = bc.Projects.GetRepository(ctx, owner, name)
 		if err != nil {
-			return nil, fmt.Errorf("unable to get repository: %w", err)
+			return nil, fmt.Errorf("%w: unable to get repository: %w", forge_types.ErrRepoNotFound, err)
 		}
 	}
 
@@ -340,7 +342,7 @@ func (c *client) Status(ctx context.Context, u *model.User, repo *model.Repo, pi
 		Duration:    uint64((pipeline.Finished - pipeline.Started) * millisecondsInSecond),
 		Parent:      common.GetPipelineStatusContext(repo, pipeline, workflow),
 		DateAdded:   bb.DateTime(time.Unix(pipeline.Started, 0)),
-		Ref:         pipeline.Ref,
+		Ref:         fmt.Sprintf("refs/heads/%s", pipeline.Branch),
 	}
 	_, err = bc.Projects.CreateBuildStatus(ctx, repo.Owner, repo.Name, pipeline.Commit, status)
 	return err
@@ -464,6 +466,11 @@ func (c *client) Deactivate(ctx context.Context, u *model.User, r *model.Repo, l
 		return fmt.Errorf("unable to create bitbucket client: %w", err)
 	}
 
+	// check repo exists
+	if _, err := c.Repo(ctx, u, r.ForgeRemoteID, r.Owner, r.Name); err != nil {
+		return fmt.Errorf("repo online check failed: %w", err)
+	}
+
 	lu, err := url.Parse(link)
 	if err != nil {
 		return err
@@ -541,7 +548,7 @@ func (c *client) getUserAndRepo(ctx context.Context, r *model.Repo) (*model.User
 		return nil, nil, fmt.Errorf("unable to get store from context")
 	}
 
-	repo, err := _store.GetRepoForgeID(r.ForgeRemoteID)
+	repo, err := _store.GetRepoForgeID(c.forgeID, r.ForgeRemoteID)
 	if err != nil {
 		return nil, nil, fmt.Errorf("unable to get repo: %w", err)
 	}
