@@ -18,7 +18,7 @@ import (
 	"encoding/base32"
 	"fmt"
 
-	"github.com/google/tink/go/subtle/random"
+	"github.com/tink-crypto/tink-go/v2/subtle/random"
 
 	"go.woodpecker-ci.org/woodpecker/v3/pipeline"
 )
